@@ -1,6 +1,6 @@
 #!/bin/bash
 #
-# Copyright (c) 2022 zfit
+# Copyright (c) 2025 zfit
 #
 
 # script has to be executed inside folder `docs`
@@ -14,6 +14,6 @@ MAKE_DOCS_PATH=$(pwd -P)
 popd >/dev/null || exit
 
 echo "Invoking docformatter"
-docformatter "${MAKE_DOCS_PATH}/../zfit/" -r --in-place --wrap-descriptions 120 --wrap-summaries 120
+docformatter "${MAKE_DOCS_PATH}/../src/zfit/" -r --in-place --wrap-descriptions 120 --wrap-summaries 120
 echo "Replacing auto docs for duplicated args"
-find "${MAKE_DOCS_PATH}/../zfit/" -type f -name '*.py' -exec "${MAKE_DOCS_PATH}/../utils/api/replace_argdocs.py" {} ';'
+find "${MAKE_DOCS_PATH}/../src/zfit/" -type f -name '*.py' -exec "${MAKE_DOCS_PATH}/../utils/api/replace_argdocs.py" {} ';'
