@@ -1,4 +1,6 @@
-#  Copyright (c) 2022 zfit
+#  Copyright (c) 2024 zfit
+from __future__ import annotations
+
 import hist
 import mplhep
 import numpy as np
@@ -13,12 +15,14 @@ for i in range(5):
     x = np.random.normal(size=1_000_000 * (i + 1)) + i**1.5 / 2 * ((-1) ** i)
     h.fill(x=x)
     histos.append(h)
-mplhep.histplot(
-    histos, stack=True, histtype="fill", label=[f"process {i + 1}" for i in range(5)]
-)
+plt.figure()
+plt.title("Histograms to be used as templates")
+mplhep.histplot(histos, stack=True, histtype="fill", label=[f"process {i + 1}" for i in range(5)])
 plt.legend()
-pdfs = [zfit.pdf.HistogramPDF(h) for h in histos]
-sumpdf = zfit.pdf.BinnedSumPDF(pdfs)
+sumpdf = zfit.pdf.BinnedSumPDF(pdfs=histos)
+# this is the shortcut for, equivalent:
+# pdfs = [zfit.pdf.HistogramPDF(h) for h in histos]
+# sumpdf = zfit.pdf.BinnedSumPDF(pdfs)
 
 
 h_back = sumpdf.to_hist()
@@ -26,6 +30,5 @@ pdf_syst = zfit.pdf.BinwiseScaleModifier(sumpdf, modifiers=True)
 
 mplhep.histplot(h_back)
 
-print(h_back)
 # uncomment to show plots
 # plt.show()
