@@ -1,9 +1,10 @@
-#  Copyright (c) 2022 zfit
+#  Copyright (c) 2024 zfit
+from __future__ import annotations
 
 import zfit
 
 # create space
-obs = zfit.Space("x", limits=(-10, 10))
+obs = zfit.Space("x", -10, 10)
 
 # parameters
 mu = zfit.Parameter("mu", 1.0, -4, 6)
@@ -16,3 +17,6 @@ gauss = zfit.pdf.Gauss(mu=mu, sigma=sigma, obs=obs)
 exponential = zfit.pdf.Exponential(lambd, obs=obs)
 
 sum_pdf = zfit.pdf.SumPDF([gauss, exponential], fracs=frac)
+
+free_param = zfit.Parameter("free", 5, 0, 15)
+param_shift = zfit.ComposedParameter("comp", lambda x: x + 5, params=free_param)
