@@ -1,15 +1,17 @@
-#  Copyright (c) 2022 zfit
+#  Copyright (c) 2025 zfit
 import pathlib
 
 import matplotlib.pyplot as plt
 import numpy as np
 import pytest
+from tqdm import tqdm
 
 
 @pytest.mark.parametrize("exact_nsample", [True, False], ids=["exact", "binomial sum"])
 def test_yield_bias(exact_nsample, ntoys=300):
     import zfit
     import zfit.z.numpy as znp
+    from zfit import z
 
     plot_folder = pathlib.Path(
         f"{'exact' if exact_nsample else 'binomial_sum'}_yield_sampling"
@@ -38,23 +40,23 @@ def test_yield_bias(exact_nsample, ntoys=300):
     def sample_func():
         exp_sample.resample()
         gauss_sample.resample()
-        gauss_val = gauss_sample.value()
-        assert gauss_val.shape[0] == true_nsig
-        exp_val = exp_sample.value()
-        assert exp_val.shape[0] == true_nbkg
-        return znp.concatenate([gauss_val, exp_val])
+        # gauss_val = gauss_sample.value()
+        # exp_val = exp_sample.value()
+        # assert gauss_val.shape[0] == true_nsig
+        # assert exp_val.shape[0] == true_nbkg
+        return zfit.data.concat([gauss_sample, exp_sample])
 
     data = model.create_sampler()
-    data.sample_holder.assign(sample_func())
+    data.update_data(sample_func())
     nll = zfit.loss.ExtendedUnbinnedNLL(model=model, data=data)
     nsigs = []
     nbkgs = []
-    minimizer = zfit.minimize.Minuit(gradient=False, tol=1e-05, mode=2)
+    minimizer = zfit.minimize.Minuit(gradient=False, tol=1e-05)
     failures = 0
-    for _ in range(ntoys):
+    for _ in tqdm(range(ntoys)):
         zfit.param.set_values(params, true_vals)
         if exact_nsample:
-            data.sample_holder.assign(sample_func())
+            data.update_data(sample_func(), guarantee_limits=True)
         else:
             data.resample(n=sum(true_vals[-2:]))
         for _ in range(10):
@@ -77,7 +79,7 @@ def test_yield_bias(exact_nsample, ntoys=300):
         nsigs.append(nsig_res)
         nbkg_res = float(result.params[n_bkg]["value"])
         nbkgs.append(nbkg_res)
-        assert nsig_res + nbkg_res == pytest.approx(true_nsig + true_nbkg, abs=0.6)
+        assert pytest.approx(true_nsig + true_nbkg, abs=0.6) == nsig_res + nbkg_res
     nsigs_mean = np.mean(nsigs)
     std_nsigs_mean = np.std(nsigs) / ntoys**0.5 * 5
     nbkg_mean = np.mean(nbkgs)
@@ -87,7 +89,8 @@ def test_yield_bias(exact_nsample, ntoys=300):
         f'{"Exact" if exact_nsample else "Binomial sum"} sampled. Fit with {minimizer.name}.'
     )
 
-    counts, edges, _ = plt.hist(nsigs, bins=50, label=" Signal yields", alpha=0.5)
+    # remove np.array if fixed: https://github.com/matplotlib/matplotlib/issues/22879
+    counts, edges, _ = plt.hist(np.array(nsigs), bins=50, label=" Signal yields", alpha=0.5)
     npoints = 50
     plt.plot(
         np.ones(npoints) * true_nsig, np.linspace(0, np.max(counts)), "gx", label="true"
@@ -117,6 +120,6 @@ def test_yield_bias(exact_nsample, ntoys=300):
     plt.legend()
     pytest.zfit_savefig(folder=plot_folder)
     rel_err_sig = 0.001
-    assert nsigs_mean == pytest.approx(true_nsig, rel=rel_err_sig, abs=std_nsigs_mean)
+    assert pytest.approx(true_nsig, rel=rel_err_sig, abs=std_nsigs_mean) == nsigs_mean
     rel_err_bkg = 0.001
-    assert nbkg_mean == pytest.approx(true_nbkg, rel=rel_err_bkg, abs=std_nbkg_mean)
+    assert pytest.approx(true_nbkg, rel=rel_err_bkg, abs=std_nbkg_mean) == nbkg_mean
