@@ -1,11 +1,12 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 import pytest
 
 
 @pytest.fixture
 def params():
     import zfit
-    return [zfit.Parameter(f'param_{i}', i) for i in range(3)]
+
+    return [zfit.Parameter(f"param_{i}", i) for i in range(3)]
 
 
 def test_approx(params):
@@ -21,7 +22,7 @@ def test_approx(params):
     np.testing.assert_allclose(approx.gradient(params=params[1]), grad[1])
     np.testing.assert_allclose(approx.gradient(params=params[2:0:-1]), grad[2:0:-1])
 
-    np.testing.assert_allclose(approx.hessian() , hessian)
+    np.testing.assert_allclose(approx.hessian(), hessian)
     assert approx.inv_hessian(invert=False) is None
     np.testing.assert_allclose(approx.inv_hessian(invert=True), np.linalg.inv(hessian))
     # now it should be available and cached
