@@ -1,15 +1,17 @@
-#  Copyright (c) 2022 zfit
+#  Copyright (c) 2025 zfit
 
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
+
 if TYPE_CHECKING:
     import zfit
 
-
 import pytest
 
+import zfit.z.numpy as znp
+
 
 @pytest.fixture
 def test_values():
@@ -36,9 +38,7 @@ def create_gauss1(nameadd=""):
     obs1 = zfit.Space("obs1", (low, high))
 
     mu = zfit.Parameter("mu" + nameadd, mu_true, mu_true - 2.0, mu_true + 7.0)
-    sigma = zfit.Parameter(
-        "sigma" + nameadd, sigma_true, sigma_true - 10.0, sigma_true + 5.0
-    )
+    sigma = zfit.Parameter("sigma" + nameadd, sigma_true, sigma_true - 10.0, sigma_true + 5.0)
 
     gauss_params1 = zfit.pdf.Gauss(mu=mu, sigma=sigma, obs=obs1, name="gauss_params1")
     return gauss_params1
@@ -61,12 +61,10 @@ def TmpGaussian():
             obs,
             mu,
             sigma,
-            dtype: type = zfit.ztypes.float,
-            name: str = "BasePDF",
             **kwargs,
         ):
             params = {"mu": mu, "sigma": sigma}
-            super().__init__(obs, params, dtype, name, **kwargs)
+            super().__init__(obs, params=params, **kwargs)
 
         def _unnormalized_pdf(self, x):
             x = x.unstack_x()
@@ -75,9 +73,7 @@ def TmpGaussian():
 
             from zfit import z
 
-            return z.exp(
-                (-((x - mu) ** 2)) / (2 * sigma**2)
-            )  # non-normalized gaussian
+            return z.exp((-((x - mu) ** 2)) / (2 * sigma**2))  # non-normalized gaussian
 
     return TmpGaussian
 
@@ -98,13 +94,8 @@ def true_gaussian_grad(x):
         / sigma_true**3
     )
     grad_sigma = (
-        -0.398942280401433
-        * np.exp(-((-mu_true + x) ** 2) / (2 * sigma_true**2))
-        / sigma_true**2
-        + 0.398942280401433
-        * (-mu_true + x) ** 2
-        * np.exp(-((-mu_true + x) ** 2) / (2 * sigma_true**2))
-        / sigma_true**4
+        -0.398942280401433 * np.exp(-((-mu_true + x) ** 2) / (2 * sigma_true**2)) / sigma_true**2
+        + 0.398942280401433 * (-mu_true + x) ** 2 * np.exp(-((-mu_true + x) ** 2) / (2 * sigma_true**2)) / sigma_true**4
     )
     return np.array((grad_mu, grad_sigma)).transpose()
 
@@ -113,9 +104,7 @@ def create_mu_sigma_2(nameadd=""):
     import zfit
 
     mu2 = zfit.Parameter("mu2" + nameadd, mu_true, mu_true - 2.0, mu_true + 7.0)
-    sigma2 = zfit.Parameter(
-        "sigma2" + nameadd, sigma_true, sigma_true - 10.0, sigma_true + 5.0
-    )
+    sigma2 = zfit.Parameter("sigma2" + nameadd, sigma_true, sigma_true - 10.0, sigma_true + 5.0)
     return mu2, sigma2
 
 
@@ -128,9 +117,7 @@ def create_wrapped_gauss(nameadd=""):
     mu2, sigma2 = create_mu_sigma_2(nameadd)
     gauss_params = dict(loc=mu2, scale=sigma2)
     tf_gauss = tfp.distributions.Normal
-    return zfit.models.dist_tfp.WrapDistribution(
-        tf_gauss, dist_params=gauss_params, obs=obs1, name="tf_gauss1"
-    )
+    return zfit.models.dist_tfp.WrapDistribution(tf_gauss, dist_params=gauss_params, obs=obs1, name="tf_gauss1")
 
 
 def create_gauss3(nameadd=""):
@@ -139,9 +126,7 @@ def create_gauss3(nameadd=""):
     obs1 = zfit.Space("obs1", (low, high))
 
     mu3 = zfit.Parameter("mu3" + nameadd, mu_true, mu_true - 2.0, mu_true + 7.0)
-    sigma3 = zfit.Parameter(
-        "sigma3" + nameadd, sigma_true, sigma_true - 10.0, sigma_true + 5.0
-    )
+    sigma3 = zfit.Parameter("sigma3" + nameadd, sigma_true, sigma_true - 10.0, sigma_true + 5.0)
     gauss3 = zfit.pdf.Gauss(mu=mu3, sigma=sigma3, obs=obs1)
     return gauss3
 
@@ -178,7 +163,7 @@ def test_input_space():
     gauss3 = create_gauss3()
     space = zfit.Space("nonexisting_obs", (-3, 5))
     with pytest.raises(ObsIncompatibleError):
-        gauss3.set_norm_range(space)
+        gauss3.pdf(x=[1.0], norm=space)
 
 
 def test_func(obs1, test_values):
@@ -196,7 +181,7 @@ def test_func(obs1, test_values):
     gauss_func = gauss_params1.as_func(norm=limits)
     vals = gauss_func.func(test_values)
     vals_pdf = gauss_params1.pdf(x=test_values, norm=limits)
-    vals, vals_pdf = zfit.run([vals, vals_pdf])
+    vals, vals_pdf = znp.asarray([vals, vals_pdf])
     np.testing.assert_allclose(vals_pdf, vals, rtol=1e-3)  # better assertion?
 
 
@@ -215,31 +200,29 @@ def test_normalization(obs1, pdf_factory):
 
     import zfit
     from zfit import z
+    import zfit.z.numpy as znp
 
     test_yield = 1524.3
     dist = pdf_factory()
-    samples = tf.cast(
-        np.random.uniform(low=low, high=high, size=100000), dtype=tf.float64
-    )
-    small_samples = tf.cast(
-        np.random.uniform(low=low, high=high, size=10), dtype=tf.float64
-    )
-    with dist.set_norm_range(zfit.Space(obs1, limits=(low, high))):
-        probs = dist.pdf(samples)
-        probs_small = dist.pdf(small_samples)
-        log_probs = dist.log_pdf(small_samples)
-        probs, log_probs = zfit.run(probs, log_probs)
-        probs = np.average(probs) * (high - low)
-        assert probs == pytest.approx(1.0, rel=0.05)
-        assert log_probs == pytest.approx(tf.math.log(probs_small).numpy(), rel=0.05)
-        dist = dist.create_extended(z.constant(test_yield))
-        probs = dist.pdf(samples)
-        probs_extended = dist.ext_pdf(samples)
-        result = probs.numpy()
-        result = np.average(result) * (high - low)
-        result_ext = np.average(probs_extended) * (high - low)
-        assert result == pytest.approx(1, rel=0.05)
-        assert result_ext == pytest.approx(test_yield, rel=0.05)
+    samples = znp.random.uniform(low=low, high=high, size=(100000,))
+
+    small_samples = znp.random.uniform(low=low, high=high, size=(10,))
+
+    norm = zfit.Space(obs1, limits=(low, high))
+
+    probs = dist.pdf(samples, norm=norm)
+    probs_small = dist.pdf(small_samples, norm=norm)
+    log_probs = dist.log_pdf(small_samples, norm=norm)
+    probs = np.average(probs) * (high - low)
+    assert pytest.approx(1.0, rel=0.05) == probs
+    assert pytest.approx(znp.log(probs_small), rel=0.05) == log_probs
+    dist = dist.create_extended(z.constant(test_yield))
+    probs = dist.pdf(samples, norm=norm)
+    probs_extended = dist.ext_pdf(samples, norm=norm)
+    result = np.average(probs) * (high - low)
+    result_ext = np.average(probs_extended) * (high - low)
+    assert pytest.approx(1, rel=0.05) == result
+    assert pytest.approx(test_yield, rel=0.05) == result_ext
 
 
 @pytest.mark.parametrize("gauss_factory", [create_gauss1, create_test_gauss1])
@@ -249,13 +232,13 @@ def test_sampling_simple(gauss_factory):
     gauss = gauss_factory()
     n_draws = 1000
     sample_tensor = gauss.sample(n=n_draws, limits=(low, high))
-    sampled_from_gauss1 = sample_tensor.numpy()
+    sampled_from_gauss1 = sample_tensor.value()
     assert max(sampled_from_gauss1[:, 0]) <= high
     assert min(sampled_from_gauss1[:, 0]) >= low
     assert n_draws == len(sampled_from_gauss1[:, 0])
     if gauss.params:
-        mu_true1 = gauss.params["mu"].numpy()
-        sigma_true1 = gauss.params["sigma"].numpy()
+        mu_true1 = gauss.params["mu"]
+        sigma_true1 = gauss.params["sigma"]
     else:
         mu_true1 = mu_true
         sigma_true1 = sigma_true
@@ -263,11 +246,11 @@ def test_sampling_simple(gauss_factory):
         n=10000,
         limits=(mu_true1 - abs(sigma_true1) * 3, mu_true1 + abs(sigma_true1) * 3),
     )
-    sampled_gauss1_full = sampled_gauss1_full.numpy()
+    sampled_gauss1_full = sampled_gauss1_full.value()
     mu_sampled = np.mean(sampled_gauss1_full)
     sigma_sampled = np.std(sampled_gauss1_full)
-    assert mu_sampled == pytest.approx(mu_true1, rel=0.07)
-    assert sigma_sampled == pytest.approx(sigma_true1, rel=0.07)
+    assert pytest.approx(mu_true1, rel=0.07) == mu_sampled
+    assert pytest.approx(sigma_true1, rel=0.07) == sigma_sampled
 
 
 def test_sampling_multiple_limits(obs1):
@@ -281,34 +264,28 @@ def test_sampling_multiple_limits(obs1):
     lower_interval = zfit.Space(obs=obs1, limits=(low1, up1))
     low2, up2 = 1, 2
     upper_interval = zfit.Space(obs=obs1, limits=(low2, up2))
-    sample_tensor = gauss_params1.sample(
-        n=n_draws, limits=lower_interval + upper_interval
-    )
-    sampled_from_gauss1 = sample_tensor.numpy()
-    between_samples = np.logical_and(
-        sampled_from_gauss1 < up1, sampled_from_gauss1 > low2
-    )
+    sample_tensor = gauss_params1.sample(n=n_draws, limits=lower_interval + upper_interval)
+    sampled_from_gauss1 = sample_tensor.value()
+    between_samples = np.logical_and(sampled_from_gauss1 < up1, sampled_from_gauss1 > low2)
     assert not any(between_samples)
     assert max(sampled_from_gauss1[:, 0]) <= up2
     assert min(sampled_from_gauss1[:, 0]) >= low1
     assert n_draws == len(sampled_from_gauss1[:, 0])
 
-    mu_true = gauss_params1.params["mu"].numpy()
-    sigma_true = gauss_params1.params["sigma"].numpy()
+    mu_true = gauss_params1.params["mu"]
+    sigma_true = gauss_params1.params["sigma"]
     low1, up1 = mu_true - abs(sigma_true) * 4, mu_true
     lower_interval = zfit.Space(obs=obs1, limits=(low1, up1))
     low2, up2 = mu_true, mu_true + abs(sigma_true) * 4
     upper_interval = zfit.Space(obs=obs1, limits=(low2, up2))
     one_interval = zfit.Space(obs=obs1, limits=(low1, up2))
 
-    sample_tensor5 = gauss_params1.sample(
-        n=10000, limits=lower_interval + upper_interval
-    )
-    sampled_gauss1_full = sample_tensor5.numpy()
+    sample_tensor5 = gauss_params1.sample(n=10000, limits=lower_interval + upper_interval)
+    sampled_gauss1_full = sample_tensor5.value()
     mu_sampled = np.mean(sampled_gauss1_full)
     sigma_sampled = np.std(sampled_gauss1_full)
-    assert mu_sampled == pytest.approx(mu_true, rel=0.07)
-    assert sigma_sampled == pytest.approx(sigma_true, rel=0.07)
+    assert pytest.approx(mu_true, rel=0.07) == mu_sampled
+    assert pytest.approx(sigma_true, rel=0.07) == sigma_sampled
 
 
 def test_analytic_sampling(obs1):
@@ -316,23 +293,23 @@ def test_analytic_sampling(obs1):
         pass
 
     import zfit
-    from zfit.core.space import ANY_UPPER
-
-    SampleGauss.register_analytic_integral(
-        func=lambda limits, params, model: 2 * limits.upper[0][0],
-        limits=zfit.Space(limits=(-float("inf"), ANY_UPPER), axes=(0,)),
-    )  # DUMMY!
-    SampleGauss.register_inverse_analytic_integral(func=lambda x, params: x + 1000.0)
+    from zfit.core.space import ANY_UPPER, ANY_LOWER
 
     mu, sigma = create_mu_sigma_true_params()
+    gauss_noana = SampleGauss(obs=obs1, mu=mu, sigma=sigma)
+
+    SampleGauss.register_analytic_integral(
+        func=lambda limits, params, model: 2 * limits.v1.upper[0],
+        limits=zfit.Space(limits=(ANY_LOWER, ANY_UPPER), axes=(0,)),
+    )  # DUMMY!
+    SampleGauss.register_inverse_analytic_integral(func=lambda x, params: x)
+
     gauss1 = SampleGauss(obs=obs1, mu=mu, sigma=sigma)
+    assert gauss_noana.has_analytic_integral
     sample = gauss1.sample(n=10000, limits=(2.0, 5.0))
-    sample.set_data_range((1002, 1005))
 
-    sample = sample.numpy()
-
-    assert 1004.0 <= min(sample[0])
-    assert 10010.0 >= max(sample[0])
+    assert 4.0 <= min(sample["obs1"])
+    assert 10.0 >= max(sample["obs1"])
 
 
 def test_multiple_limits(obs1):
@@ -346,10 +323,7 @@ def test_multiple_limits(obs1):
     multiple_limits_upper = ((1.1,), (2.1,), (9.1,))
 
     multiple_limits_range = MultiSpace(
-        [
-            zfit.Space(limits=(low, up), axes=dims)
-            for low, up in zip(multiple_limits_lower, multiple_limits_upper)
-        ]
+        [zfit.Space(limits=(low, up), axes=dims) for low, up in zip(multiple_limits_lower, multiple_limits_upper)]
     )
     integral_simp = gauss_params1.integrate(
         limits=simple_limits,
@@ -359,27 +333,17 @@ def test_multiple_limits(obs1):
         limits=multiple_limits_range,
         norm=False,
     )
-    integral_simp_num = gauss_params1.numeric_integrate(
-        limits=simple_limits, norm=False
-    )
-    integral_mult_num = gauss_params1.numeric_integrate(
-        limits=multiple_limits_range, norm=False
-    )
+    integral_simp_num = gauss_params1.numeric_integrate(limits=simple_limits, norm=False)
+    integral_mult_num = gauss_params1.numeric_integrate(limits=multiple_limits_range, norm=False)
 
-    integral_simp, integral_mult = [integral_simp.numpy(), integral_mult.numpy()]
+    integral_simp, integral_mult = [integral_simp, integral_mult]
     integral_simp_num, integral_mult_num = [
-        integral_simp_num.numpy(),
-        integral_mult_num.numpy(),
+        integral_simp_num,
+        integral_mult_num,
     ]
-    assert integral_simp == pytest.approx(
-        integral_mult, rel=1e-2
-    )  # big tol as mc is used
-    assert integral_simp == pytest.approx(
-        integral_simp_num, rel=1e-2
-    )  # big tol as mc is used
-    assert integral_simp_num == pytest.approx(
-        integral_mult_num, rel=1e-2
-    )  # big tol as mc is used
+    assert pytest.approx(integral_mult, rel=1e-2) == integral_simp  # big tol as mc is used
+    assert pytest.approx(integral_simp_num, rel=1e-2) == integral_simp  # big tol as mc is used
+    assert pytest.approx(integral_mult_num, rel=1e-2) == integral_simp_num  # big tol as mc is used
 
 
 def test_copy(obs1):
@@ -410,10 +374,12 @@ def test_projection_pdf(test_values):
     import numpy as np
 
     import zfit
+
+    # zfit.run.set_graph_mode(False)
     import zfit.z.numpy as znp
 
-    x = zfit.Space("x", limits=(-1, 1))
-    y = zfit.Space("y", limits=(-1, 1))
+    x = zfit.Space("x", -1, 1)
+    y = zfit.Space("y", -1, 1)
 
     def correlated_func(self, x):
         x, y = x.unstack_x()
@@ -421,34 +387,26 @@ def test_projection_pdf(test_values):
         return value
 
     def correlated_func_integrate_x(y, limits):
-        lower, upper = limits.rect_limits
+        lower, upper = limits.v1.limits
 
         def integ(x, y):
-            return (
-                0.333333333333333 * x**3
-                - 1.0 * x**2 * y**3
-                + x * (1.0 * y**6 + 0.1)
-            )
+            return 0.333333333333333 * x**3 - 1.0 * x**2 * y**3 + x * (1.0 * y**6 + 0.1)
 
         return integ(y, upper) - integ(y, lower)
 
     def correlated_func_integrate_y(x, limits):
-        lower, upper = limits.rect_limits
+        lower, upper = limits.v1.limits
 
         def integ(x, y):
-            return (
-                -0.5 * x * y**4
-                + 0.142857142857143 * y**7
-                + y * (1.0 * x**2 + 0.1)
-            )
+            return -0.5 * x * y**4 + 0.142857142857143 * y**7 + y * (1.0 * x**2 + 0.1)
 
-        return (integ(x, upper) - integ(x, lower))[0]
+        return integ(x, upper) - integ(x, lower)
 
     obs = x * y
     from zfit.models.special import SimplePDF
 
     gauss_xy = SimplePDF(func=correlated_func, obs=obs)
-    assert gauss_xy.create_projection_pdf(limits=y).norm_range == x
+    assert gauss_xy.create_projection_pdf(limits=y).norm == x
     proj_pdf = gauss_xy.create_projection_pdf(limits=y)
     test_values = znp.array(
         [
@@ -479,5 +437,34 @@ def test_projection_pdf(test_values):
         norm=False,
     )
     probs = proj_pdf.pdf(x=test_values)
-    probs = probs.numpy()
     np.testing.assert_allclose(probs, true_probs, rtol=1e-3)  # MC normalization
+
+
+def test_invalid_pdf_names():
+    """Test that invalid PDF names result in a warning."""
+    import zfit
+    import pytest
+
+    obs = zfit.Space("obs1", (low, high))
+    mu = zfit.Parameter("mu", 0.0)
+    sigma = zfit.Parameter("sigma", 1.0)
+
+    # Test name with invalid characters
+    with pytest.warns(DeprecationWarning, match="Name contains invalid characters"):
+        pdf_invalid_chars = zfit.pdf.Gauss(mu=mu, sigma=sigma, obs=obs, name="invalid name")
+
+    # Test name starting with "-"
+    with pytest.warns(DeprecationWarning, match="Name cannot start with '-'"):
+        pdf_starts_with_dash = zfit.pdf.Gauss(mu=mu, sigma=sigma, obs=obs, name="-invalid")
+
+    # Test name starting with "."
+    with pytest.warns(DeprecationWarning, match="Name cannot start with '.'"):
+        pdf_starts_with_dot = zfit.pdf.Gauss(mu=mu, sigma=sigma, obs=obs, name=".invalid")
+
+    # Test reserved word
+    with pytest.warns(DeprecationWarning, match="is a reserved word"):
+        pdf_reserved = zfit.pdf.Gauss(mu=mu, sigma=sigma, obs=obs, name="null")
+
+    # Test that valid names don't raise warnings
+    pdf_valid = zfit.pdf.Gauss(mu=mu, sigma=sigma, obs=obs, name="valid_name")
+    assert pdf_valid.name == "valid_name"
