@@ -1,16 +1,17 @@
-#  Copyright (c) 2020 zfit
+#  Copyright (c) 2022 zfit
 
 import copy
 
 import pytest
 
 import zfit
-# noinspection PyUnresolvedReferences
-from zfit.core.testing import setup_function, teardown_function, tester
+
 
 @pytest.mark.first  # needs to run before initialization
-@pytest.mark.parametrize(["n_cpu", 'taken', 'left'], [[3, 5, 0], [0, -1, 0], [10, 3, 7],
-                                                      [5, -1, 0], [8, -3, 2], [5, 0, 5]])
+@pytest.mark.parametrize(
+    ["n_cpu", "taken", "left"],
+    [[3, 5, 0], [0, -1, 0], [10, 3, 7], [5, -1, 0], [8, -3, 2], [5, 0, 5]],
+)
 def test_cpu_management(n_cpu, taken, left):
     zfit.run.set_n_cpu(n_cpu=n_cpu)
     _cpu = copy.deepcopy(zfit.run._cpu)
