@@ -1,4 +1,4 @@
-#  Copyright (c) 2022 zfit
+#  Copyright (c) 2024 zfit
 
 import numpy as np
 import pytest
@@ -27,8 +27,8 @@ def test_extract_extended_pdfs():
     sum3 = zfit.pdf.SumPDF((gauss5, gauss6), 0.4)
     sum3_ext = sum3.create_extended(yield1)
 
-    sum_all = zfit.pdf.SumPDF(pdfs=[sum2_ext_daughters, sum3_ext])
-    sum_all.set_norm_range((-5, 5))
+    sum_all = zfit.pdf.SumPDF(pdfs=[sum2_ext_daughters, sum3_ext], norm=(-5, 5))
+
 
     extracted_pdfs = extract_extended_pdfs(pdfs=sum_all)
     assert frozenset(extracted_pdfs) == {gauss3_ext, gauss4_ext, sum3_ext}
@@ -36,15 +36,13 @@ def test_extract_extended_pdfs():
     limits = zfit.Space(obs=obs1, limits=(-4, 5))
     limits = limits.with_autofill_axes()
     extended_sample = extended_sampling(pdfs=sum_all, limits=limits)
-    extended_sample_np = extended_sample.numpy()
-    assert np.shape(extended_sample_np)[0] == pytest.approx(
+    assert pytest.approx(
         expected=(45 + 100 + 200), rel=0.1
-    )
-    samples_from_pdf = sum_all.sample(n="extended", limits=limits)
-    samples_from_pdf_np = samples_from_pdf.numpy()
-    assert np.shape(samples_from_pdf_np)[0] == pytest.approx(
+    ) == np.shape(extended_sample)[0]
+    samples_from_pdf = sum_all.sample(n="extended", limits=limits).value()
+    assert pytest.approx(
         expected=(45 + 100 + 200), rel=0.1
-    )
+    ) == np.shape(samples_from_pdf)[0]
 
 
 def test_set_yield():
