@@ -1,4 +1,5 @@
-#  Copyright (c) 2022 zfit
+#  Copyright (c) 2025 zfit
+import jacobi
 import numpy as np
 import pytest
 import tensorflow as tf
@@ -7,6 +8,7 @@ import zfit.core.basepdf
 import zfit.models.dist_tfp
 import zfit.settings
 from zfit import z
+import zfit.z.numpy as znp
 from zfit.core.loss import UnbinnedNLL
 from zfit.core.space import Space
 from zfit.minimize import Minuit
@@ -23,7 +25,7 @@ test_values_np = np.random.normal(loc=mu_true, scale=sigma_true, size=(yield_tru
 
 
 def create_test_values(size):
-    return tf.random.normal(mean=mu_true, stddev=sigma_true, shape=(size, 1))
+    return z.random.get_prng().normal(mean=mu_true, stddev=sigma_true, shape=(size, 1))
 
 
 test_values_np2 = np.random.normal(loc=mu_true2, scale=sigma_true2, size=yield_true)
@@ -145,13 +147,13 @@ def test_extended_unbinned_nll(size):
     minimizer = Minuit(tol=1e-4)
     status = minimizer.minimize(loss=nll)
     params = status.params
-    assert params[mu3]["value"] == pytest.approx(
-        zfit.run(tf.math.reduce_mean(test_values.value())), rel=0.05
-    )
-    assert params[sigma3]["value"] == pytest.approx(
-        zfit.run(tf.math.reduce_std(test_values.value())), rel=0.05
-    )
-    assert params[yield3]["value"] == pytest.approx(size, rel=0.005)
+    assert pytest.approx(
+        np.mean(test_values.value()), rel=0.05
+    ) == params[mu3]["value"]
+    assert pytest.approx(
+        np.std(test_values.value()), rel=0.05
+    ) == params[sigma3]["value"]
+    assert pytest.approx(size, rel=0.005) == params[yield3]["value"]
 
 
 def test_unbinned_simultaneous_nll():
@@ -161,10 +163,10 @@ def test_unbinned_simultaneous_nll():
     params = status.params
     assert set(nll.get_params()) == {mu1, mu2, sigma1, sigma2}
 
-    assert params[mu1]["value"] == pytest.approx(np.mean(test_values_np), rel=0.007)
-    assert params[mu2]["value"] == pytest.approx(np.mean(test_values_np2), rel=0.007)
-    assert params[sigma1]["value"] == pytest.approx(np.std(test_values_np), rel=0.007)
-    assert params[sigma2]["value"] == pytest.approx(np.std(test_values_np2), rel=0.007)
+    assert pytest.approx(np.mean(test_values_np), rel=0.007) == params[mu1]["value"]
+    assert pytest.approx(np.mean(test_values_np2), rel=0.007) == params[mu2]["value"]
+    assert pytest.approx(np.std(test_values_np), rel=0.007) == params[sigma1]["value"]
+    assert pytest.approx(np.std(test_values_np2), rel=0.007) == params[sigma2]["value"]
 
 
 @pytest.mark.flaky(3)
@@ -188,15 +190,15 @@ def test_unbinned_nll(weights, sigma, options):
     params = status.params
     rel_error = 0.12 if weights is None else 0.1  # more fluctuating with weights
 
-    assert params[mu1]["value"] == pytest.approx(np.mean(test_values_np), rel=rel_error)
-    assert params[sigma1]["value"] == pytest.approx(
+    assert pytest.approx(np.mean(test_values_np), rel=rel_error) == params[mu1]["value"]
+    assert pytest.approx(
         np.std(test_values_np), rel=rel_error
-    )
+    ) == params[sigma1]["value"]
 
-    constraints = zfit.constraint.nll_gaussian(
+    constraints = zfit.constraint.GaussianConstraint(
         params=[mu2, sigma2],
         observation=[mu_constr[0], sigma_constr[0]],
-        uncertainty=sigma(),
+        cov=sigma(),
     )
     nll_object = UnbinnedNLL(
         model=gaussian2, data=test_values, constraints=constraints, options=options
@@ -233,11 +235,11 @@ def test_add():
     ranges[2] = Space(limits=(3, 6), obs=obs1)
     ranges[3] = Space(limits=(4, 7), obs=obs1)
 
-    constraint1 = zfit.constraint.nll_gaussian(
-        params=param1, observation=1.0, uncertainty=0.5
+    constraint1 = zfit.constraint.GaussianConstraint(
+        params=param1, observation=1.0, sigma=0.5
     )
-    constraint2 = zfit.constraint.nll_gaussian(
-        params=param3, observation=2.0, uncertainty=0.25
+    constraint2 = zfit.constraint.GaussianConstraint(
+        params=param3, observation=2.0, sigma=0.25
     )
     merged_contraints = [constraint1, constraint2]
 
@@ -267,15 +269,15 @@ def test_add():
     assert simult_nll.fit_range == ranges
 
     def eval_constraint(constraints):
-        return z.reduce_sum([c.value() for c in constraints]).numpy()
+        return z.reduce_sum([c.value() for c in constraints])
 
     assert eval_constraint(simult_nll.constraints) == eval_constraint(merged_contraints)
     assert set(simult_nll.get_params()) == {param1, param2, param3}
 
 
-# @pytest.mark.xfail  # TODO(TF2): grads not supported, use numerical ones? Or calculate on the fly?
+@pytest.mark.parametrize("numgrad", [True, False], ids=["numgrad", "autograd"])
 @pytest.mark.parametrize("chunksize", [10000000, 1000])
-def test_gradients(chunksize):
+def test_gradients(chunksize, numgrad):
     from numdifftools import Gradient
 
     zfit.run.chunking.active = True
@@ -287,46 +289,50 @@ def test_gradients(chunksize):
     param2 = zfit.Parameter("param2", initial2)
 
     gauss1 = Gauss(param1, 4, obs=obs1)
-    gauss1.set_norm_range((-5, 5))
     gauss2 = Gauss(param2, 5, obs=obs1)
-    gauss2.set_norm_range((-5, 5))
 
     data1 = zfit.Data.from_tensor(obs=obs1, tensor=z.constant(1.0, shape=(100,)))
-    data1.set_data_range((-5, 5))
     data2 = zfit.Data.from_tensor(obs=obs1, tensor=z.constant(1.0, shape=(100,)))
-    data2.set_data_range((-5, 5))
 
     nll = UnbinnedNLL(model=[gauss1, gauss2], data=[data1, data2])
 
-    def loss_func(values):
-        for val, param in zip(values, nll.get_cache_deps(only_floating=True)):
-            param.set_value(val)
-        return nll.value().numpy()
+    def loss_funcparam1(values):
+        with zfit.param.set_values(param2, values):
+            return nll.value()
 
-    # theoretical, numerical = tf.test.compute_gradient(loss_func, list(params))
-    gradient1 = nll.gradient(params=param1)
-    gradient_func = Gradient(loss_func)
-    # gradient_func = lambda *args, **kwargs: list(gradient_func_numpy(*args, **kwargs))
-    assert gradient1[0].numpy() == pytest.approx(gradient_func([param1.numpy()]))
+    def loss_funcparam1and2(values):
+        with zfit.param.set_values([param2, param1], values):
+            return nll.value()
+
+    gradient_func_num = Gradient(loss_funcparam1)
+
+    gradient1_num = gradient_func_num([param2])
+    gradient1 = nll.gradient(params=param2, numgrad=numgrad)
+    grad_num_truth = jacobi.jacobi(loss_funcparam1, [param2])[0]
+    np.testing.assert_allclose(gradient1, grad_num_truth, rtol=1e-6)
+    np.testing.assert_allclose(gradient1, gradient1_num, rtol=1e-6)
     param1.set_value(initial1)
     param2.set_value(initial2)
     params = [param2, param1]
-    gradient2 = nll.gradient(params=params)
-    both_gradients_true = list(
-        reversed(list(gradient_func([initial1, initial2])))
-    )  # because param2, then param1
-    assert [g.numpy() for g in gradient2] == pytest.approx(both_gradients_true)
+    gradient2 = nll.gradient(params=params, numgrad=numgrad)
+    gradient_func1and2 = Gradient(loss_funcparam1and2, order=2, base_step=0.1)
+    gradient2_true_numdiff = gradient_func1and2([initial2, initial1])
+    gradient2_true = jacobi.jacobi(loss_funcparam1and2, [initial2, initial1])[0]
+    np.testing.assert_allclose(gradient2_true, gradient2_true_numdiff, rtol=1e-6)  # if this fails, numdiff/jacobi disagree
+    np.testing.assert_allclose(gradient2, gradient2_true, rtol=1e-5)
 
     param1.set_value(initial1)
     param2.set_value(initial2)
-    gradient3 = nll.gradient()
+
+    params3 = nll.get_params()
+    gradient3 = nll.gradient(params3)
     gradients_true3 = []
-    for param_o in nll.get_params():
+    for param_o in params3:
         for param, grad in zip(params, gradient2):
             if param_o is param:
                 gradients_true3.append(float(grad))
                 break
-    assert [g.numpy() for g in gradient3] == pytest.approx(gradients_true3)
+    np.testing.assert_allclose(gradient3, gradients_true3, rtol=1e-5)
 
 
 def test_simple_loss():
@@ -335,7 +341,7 @@ def test_simple_loss():
     true_c = -0.3
     truevals = true_a, true_b, true_c
     a_param = zfit.Parameter(
-        "variable_a15151loss", 1.5, -1.0, 20.0, step_size=z.constant(0.1)
+        "variable_a15151loss", 1.5, -1.0, 20.0, stepsize=z.constant(0.1)
     )
     b_param = zfit.Parameter("variable_b15151loss", 3.5)
     c_param = zfit.Parameter("variable_c15151loss", -0.23)
@@ -358,36 +364,77 @@ def test_simple_loss():
 
     loss_func.errordef = 1
     loss_deps = zfit.loss.SimpleLoss(func=loss_func, params=param_list)
-    # loss = zfit.loss.SimpleLoss(func=loss_func)
     loss = zfit.loss.SimpleLoss(func=loss_func, params=param_list)
     loss2 = zfit.loss.SimpleLoss(func=loss_func, params=truevals)
 
-    assert loss_deps.get_cache_deps() == set(param_list)
     assert set(loss_deps.get_params()) == set(param_list)
 
     loss_tensor = loss_func(param_list)
-    loss_value_np = loss_tensor.numpy()
+    loss_value_np = loss_tensor
 
-    assert loss.value().numpy() == pytest.approx(loss_value_np)
-    assert loss_deps.value().numpy() == pytest.approx(loss_value_np)
+    assert pytest.approx(loss_value_np) == loss.value()
+    assert pytest.approx(loss_value_np) == loss_deps.value()
 
-    with pytest.raises(IntentionAmbiguousError):
-        _ = loss + loss_deps
+    assert pytest.approx(
+        loss_deps.value(full=True)
+    ) == loss.value(full=True)
 
     minimizer = zfit.minimize.Minuit()
     result = minimizer.minimize(loss=loss)
     assert result.valid
-    assert true_a == pytest.approx(result.params[a_param]["value"], rel=0.03)
-    assert true_b == pytest.approx(result.params[b_param]["value"], rel=0.06)
-    assert true_c == pytest.approx(result.params[c_param]["value"], rel=0.5)
+    assert pytest.approx(result.params[a_param]["value"], rel=0.03) == true_a
+    assert pytest.approx(result.params[b_param]["value"], rel=0.06) == true_b
+    assert pytest.approx(result.params[c_param]["value"], rel=0.5) == true_c
 
-    zfit.param.set_values(param_list, np.array(zfit.run(param_list)) + 0.6)
+    zfit.param.set_values(param_list, znp.asarray(param_list) + 0.6)
     result2 = minimizer.minimize(loss=loss2)
     assert result2.valid
     params = list(result2.params)
-    assert true_a == pytest.approx(result2.params[params[0]]["value"], rel=0.03)
-    assert true_b == pytest.approx(result2.params[params[1]]["value"], rel=0.06)
-    assert true_c == pytest.approx(result2.params[params[2]]["value"], rel=0.5)
+    assert pytest.approx(result2.params[params[0]]["value"], rel=0.03) == true_a
+    assert pytest.approx(result2.params[params[1]]["value"], rel=0.06) == true_b
+    assert pytest.approx(result2.params[params[2]]["value"], rel=0.5) == true_c
+
+
+def test_simple_loss_addition():
+
+    pa = zfit.Parameter("pa", 1.0)
+    pb = zfit.Parameter("pb", 2.0)
+    pc = zfit.Parameter("pc", 3.0)
+    pd = zfit.Parameter("pd", 4.0)
+    pe = zfit.Parameter("pe", 5.0)
+
+    loss1 = zfit.loss.SimpleLoss(lambda x: x[0] ** 2, params=pa, errordef=0.5)
+    loss2 = zfit.loss.SimpleLoss(lambda x: x[0] ** 2 + x[1] ** 2, params=[pb, pc], errordef=0.5)
+    loss3 = zfit.loss.SimpleLoss(lambda x: x[0] ** 2 + x[1] ** 2 + x[2] ** 2, params=[pa, pb, pc], errordef=2)
+    loss4 = zfit.loss.SimpleLoss(lambda x: x[0] ** 2 + x[1] ** 2, [pd, pe], errordef=0.5)
+
+    loss12 = loss1 + loss2
+
+    assert set(loss12.get_params()) == {pa, pb, pc}
+    assert loss12.errordef == 0.5
+    assert loss12.value() == loss1.value() + loss2.value()
+    assert loss12.value(full=True) == loss1.value(full=True) + loss2.value(full=True)
+
+    loss123 = loss1 + loss2 + loss3
+    assert set(loss123.get_params()) == {pa, pb, pc}
+    assert loss123.errordef == 0.5
+    assert loss123.value() == loss1.value() + loss2.value() + 0.25 * loss3.value()
+    assert loss123.value(full=True) == loss1.value(full=True) + loss2.value(full=True) + 0.25 * loss3.value(full=True)
+
+    loss1234 = loss1 + loss2 + loss3 + loss4
+    assert set(loss1234.get_params()) == {pa, pb, pc, pd, pe}
+    assert loss1234.errordef == 0.5
+    assert loss1234.value() == loss1.value() + loss2.value() + 0.25 * loss3.value() + loss4.value()
+    assert loss1234.value(full=True) == loss1.value(full=True) + loss2.value(full=True) + 0.25 * loss3.value(full=True) + loss4.value(full=True)
+
+    loss3124 = loss3 + loss1 + loss2 + loss4
+    assert set(loss3124.get_params()) == {pa, pb, pc, pd, pe}
+    assert loss3124.errordef == 2
+    assert loss3124.value() ==  loss3.value() + 4 * loss1.value() + 4 * loss2.value() + 4 * loss4.value()
+    assert loss3124.value(full=True) == loss3.value(full=True) + 4 * loss1.value(full=True) + 4 * loss2.value(full=True) + 4 * loss4.value(full=True)
+
+
+
 
 
 def test_create_new_nll():
@@ -422,7 +469,7 @@ def test_create_new_extnll():
     nll = zfit.loss.ExtendedUnbinnedNLL(
         model=gaussian1,
         data=test_values,
-        constraints=zfit.constraint.GaussianConstraint(mu1, 1.0, 0.1),
+        constraints=zfit.constraint.GaussianConstraint(mu1, 1.0, sigma=0.1),
     )
 
     nll2 = nll.create_new(model=gaussian1)
@@ -458,13 +505,14 @@ def test_callable_loss(create_loss):
     loss = create_loss()
 
     params = list(loss.get_params())
-    x = np.array(zfit.run(params)) + 0.1
+    x = np.array(znp.asarray(params)) + 0.1
     value_loss = loss(x)
     with zfit.param.set_values(params, x):
-        true_val = zfit.run(loss.value())
-        assert true_val == pytest.approx(zfit.run(value_loss))
+        true_val = zfit.run(loss.value(full=True))
+        _ = zfit.run(loss.value(full=True))
+        assert pytest.approx(value_loss) == true_val
         with pytest.raises(BehaviorUnderDiscussion):
-            assert true_val == pytest.approx(zfit.run(loss()))
+            assert pytest.approx(loss()) == true_val
 
     with pytest.raises(ValueError):
         loss(x[:-1])
@@ -479,7 +527,7 @@ def test_iminuit_compatibility(create_loss):
     loss = create_loss()
 
     params = list(loss.get_params())
-    x = np.array(zfit.run(params)) + 0.1
+    x = znp.asarray(params) + 0.1
     zfit.param.set_values(params, x)
 
     with pytest.raises(ValueError):
@@ -497,13 +545,14 @@ def test_iminuit_compatibility(create_loss):
     zfit.param.set_values(params, x)
     minimizer_zfit = zfit.minimize.Minuit()
     result_zfit = minimizer_zfit.minimize(loss)
-    assert result_zfit.fmin == pytest.approx(result.fmin.fval, abs=0.03)
+    assert pytest.approx(result.fmin.fval, abs=0.03) == result_zfit.fmin
 
 
-# @pytest.mark.skip  # TODO: reactivate and make real, just mockup
 @pytest.mark.flaky(3)
-# @pytest.mark.parametrize('weights', [None, np.random.normal(loc=1., scale=0.2, size=test_values_np.shape[0])])
-@pytest.mark.parametrize("weights", [None])
+@pytest.mark.parametrize(
+    "weights", [None, np.random.normal(loc=1.0, scale=0.1, size=10000)]
+)
+# @pytest.mark.parametrize("weights", [None])
 def test_binned_nll(weights):
     obs = zfit.Space("obs1", limits=(-15, 25))
     gaussian1, mu1, sigma1 = create_gauss1(obs=obs)
@@ -521,10 +570,10 @@ def test_binned_nll(weights):
     params = status.params
     rel_error = 0.035 if weights is None else 0.15  # more fluctuating with weights
 
-    assert params[mu1]["value"] == pytest.approx(np.mean(test_values_np), rel=rel_error)
-    assert params[sigma1]["value"] == pytest.approx(
+    assert pytest.approx(np.mean(test_values_np), rel=rel_error) == params[mu1]["value"]
+    assert pytest.approx(
         np.std(test_values_np), rel=rel_error
-    )
+    ) == params[sigma1]["value"]
 
     constraints = zfit.constraint.GaussianConstraint(
         params=[mu2, sigma2],
@@ -543,3 +592,20 @@ def test_binned_nll(weights):
     if weights is None:
         assert params[mu2]["value"] > np.mean(test_values_np)
         assert params[sigma2]["value"] < np.std(test_values_np)
+
+
+def test_loss_from_data_outside_fails():
+
+    obs = zfit.Space("obs1", limits=(-15, 25))
+    pdf = zfit.pdf.Gauss(1.0, 1.0, obs=obs)
+    data = np.linspace(-20, 20, 1000)
+
+    with pytest.raises(IntentionAmbiguousError):
+        zfit.loss.UnbinnedNLL(model=pdf, data=data)
+
+
+
+    loss = zfit.loss.UnbinnedNLL(model=pdf, data=zfit.Data(data, obs=obs.obs))
+    assert loss.value() ** 2 > -1  # just a sanity check
+    loss2 = zfit.loss.UnbinnedNLL(model=pdf, data=zfit.Data(data, obs=obs))
+    assert loss2.value() ** 2 > -1  # just a sanity check
