@@ -1,7 +1,7 @@
-#  Copyright (c) 2022 zfit
-import tensorflow as tf
+#  Copyright (c) 2024 zfit
 
 import zfit
+from zfit import z
 
 mean1_true = 1.0
 width1_true = 1.4
@@ -14,16 +14,14 @@ obs1 = zfit.Space(obs1, limits=limits1)
 
 
 def test_breitwigner1():
-    test_values = tf.random.uniform(minval=-2, maxval=4, shape=(100,))
+    test_values = z.random.uniform(minval=-2, maxval=4, shape=(100,))
     mean = zfit.Parameter("mean", mean1_true, -10, 10)
     width = zfit.Parameter("width", width1_true)
     bw1 = zfit.pdf.Cauchy(m=mean, gamma=width, obs=obs1)
 
     probs1 = bw1.pdf(x=test_values)
-    # TODO: add scipy dist?
-    probs1 = probs1.numpy()
     assert all(probs1) > 0
     # np.testing.assert_allclose(probs1, probs1_tfp, rtol=1e-2)
 
     sample1 = bw1.sample(100)
-    assert len(sample1.numpy()) == 100
+    assert len(sample1) == 100
