@@ -1,4 +1,7 @@
-#  Copyright (c) 2022 zfit
+#  Copyright (c) 2024 zfit
+import hist.axis
+import numpy as np
+import pandas as pd
 import pytest
 import tensorflow as tf
 
@@ -92,7 +95,9 @@ limits_to_test = [
     [{"multi": [limits1, limits2any]}, {"multi": [limits1any, limits2]}],
 ]
 
-
+def test_no_binning_transform():
+    with pytest.raises(ValueError):
+        _ = zfit.Space("x",binning=hist.axis.Regular(1, 1, 2, name='x', transform=hist.axis.transform.log))
 def test_illegal_bounds():
     with pytest.raises(tf.errors.InvalidArgumentError):
         _ = zfit.Space(["obs1", "obs2"], ([-1, 4], [2, 3]))
@@ -139,15 +144,15 @@ def test_rect_limits():
     assert space1.has_rect_limits
     space1_lim = space1_nolim.with_limits((0, 1))
     space1_ax = Space(axes=0, limits=(0, 1))
-    lower, upper = space1.rect_limits
+    lower, upper = space1.v1.limits
     assert lower == 0
     assert upper == 1
 
-    lower, upper = space1_ax.rect_limits
+    lower, upper = space1_ax.v1.limits
     assert lower == 0
     assert upper == 1
 
-    lower, upper = space1_lim.rect_limits
+    lower, upper = space1_lim.v1.limits
     assert lower == 0
     assert upper == 1
 
@@ -247,7 +252,7 @@ def test_with_coords(limits):
 
 @pytest.mark.parametrize("binning", [None, [binning1, binning2]])
 @pytest.mark.parametrize("limits", limits_to_test)
-def test_with_obs(limits, binning):
+def test_with_obs_binning_limits1(limits, binning):
     """
     Args:
         limits:
@@ -264,13 +269,13 @@ def test_with_obs(limits, binning):
     space1axes = space_factory(limits=limits1, axes=axes1)
     using_space_as_lim = isinstance(limits1, Space)
     if isinstance(limits1, Space):
-        limits1 = limits1.rect_limits
+        limits1 = limits1.v1.limits
     space1mixed = space_factory(obs1, limits=limits1, axes=axes2, binning=binning1)
 
     space2obs = space_factory(obs2, limits=limits2, binning=binning2)
     space2 = space_factory(obs2, limits=limits2, axes=axes2, binning=binning2)
     if isinstance(limits2, Space):
-        limits2 = limits2.rect_limits
+        limits2 = limits2.v1.limits
     space2axes = space_factory(limits=limits2, axes=axes2)
     space2mixed = space_factory(obs2, limits=limits1, axes=axes1, binning=binning2)
 
@@ -301,7 +306,6 @@ def test_with_obs(limits, binning):
 
         space = space_used.with_obs(obs2)
         if using_space_as_lim:
-
             assert space == space2mixed
         else:
             assert space == space2mixed
@@ -320,12 +324,12 @@ def test_with_axes(limits):
     space1axes = space_factory(limits=limits1, axes=axes1)
     using_space_as_lim = isinstance(limits1, Space)
     if isinstance(limits1, Space):
-        limits1 = limits1.rect_limits
+        limits1 = limits1.v1.limits
     space1mixed = space_factory(obs1, limits=limits1, axes=axes2)
 
     space2 = space_factory(obs2, limits=limits2, axes=axes2)
     if isinstance(limits2, Space):
-        limits2 = limits2.rect_limits
+        limits2 = limits2.v1.limits
     space2axes = space_factory(limits=limits2, axes=axes2)
 
     # define which space to use in this tests
@@ -371,7 +375,7 @@ def test_space_add(limits):
     space1 = space_factory(obs1, limits=limits1, axes=axes1)
     space1axes = space_factory(limits=limits1, axes=axes1)
     if isinstance(limits1, Space):
-        limits1 = limits1.rect_limits
+        limits1 = limits1.v1.limits
     space1mixed = space_factory(obs1, limits=limits1, axes=axes2)
 
     space2obs = space_factory(obs2, limits=limits2)
@@ -379,7 +383,7 @@ def test_space_add(limits):
 
     space2axes = space_factory(limits=limits2, axes=axes2)
     if isinstance(limits2, Space):
-        limits2 = limits2.rect_limits
+        limits2 = limits2.v1.limits
     space2mixed = space_factory(obs2, limits=limits2, axes=axes1)
 
     space = space1 + space2
@@ -506,20 +510,20 @@ def test_create_binned_space():
     binning = zfit.binned.RegularBinning(50, -10, 10, name="x")
     obs_bin = zfit.Space("x", binning=binning)
     assert obs_bin.binning["x"] == binning
-    assert obs_bin.lower[0][0] == -10
-    assert obs_bin.upper[0][0] == 10
+    assert obs_bin.v1.lower[0] == -10
+    assert obs_bin.v1.upper[0] == 10
 
     binning_n = 50
     obs2 = zfit.Space("x", limits=(-10, 10), binning=binning_n)
     assert obs2.binning["x"] == binning
-    assert obs2.lower[0][0] == -10
-    assert obs2.upper[0][0] == 10
+    assert obs2.v1.lower[0] == -10
+    assert obs2.v1.upper[0] == 10
 
     binning = zfit.binned.VariableBinning([1, 5, 7.3, 11.2], name="x")
     obs_bin = zfit.Space("x", binning=binning)
     assert obs_bin.binning["x"] == binning
-    assert obs_bin.lower[0][0] == 1
-    assert obs_bin.upper[0][0] == 11.2
+    assert obs_bin.v1.lower[0] == 1
+    assert obs_bin.v1.upper[0] == 11.2
 
 
 def test_create_binned_raises():
@@ -537,3 +541,79 @@ def test_create_binned_raises():
         _ = zfit.Space("x", binning=binning)
     with pytest.raises(ObsIncompatibleError):
         _ = zfit.Space(["x", "z"], binning=binning)
+
+
+def test_compat_layer_space():
+    space1 = zfit.Space("x", -1, 1)
+    space2 = zfit.Space("y", -3, 5)
+    space = space1 * space2
+    assert space.v1.lower[0] == -1
+    assert space.v1.upper[0] == 1
+    assert space.v1.lower[1] == -3
+    assert space.v1.upper[1] == 5
+
+def test_filter_pandas_space():
+    df = pd.DataFrame({"x": np.random.normal(0, 1, 1000), "y": np.random.normal(-100, 1, 1000)})
+    space1 = zfit.Space("x", -1, 1)
+    space2 = zfit.Space("y", -100, -99)
+    space = space2 * space1
+    df = space.filter(df)
+    assert df["x"].min() >= -1
+    assert df["x"].max() <= 1
+    assert df["y"].min() >= -100
+    assert df["y"].max() <= -99
+
+def test_vectorized_limits_v1_dummy():
+    import zfit.z.numpy as znp
+
+    obs1 = zfit.Space("x", -1, 1)
+    obs2 = zfit.Space("y", -3, 5)
+    space = obs1 * obs2
+
+    assert space.v1.lower[0] == -1
+    assert space.v1.upper[0] == 1
+    assert space.v1.lower[1] == -3
+    assert space.v1.upper[1] == 5
+
+    assert space.vec.lower.shape == (1, 2)
+    assert space.vec.upper.shape == (1, 2)
+    assert space.vec.lower[0, 0] == -1
+    assert space.vec.upper[0, 0] == 1
+    assert space.vec.lower[0, 1] == -3
+    assert space.vec.upper[0, 1] == 5
+
+    np.testing.assert_array_equal(space.v1.limits, ([-1, -3], [1, 5]))
+    np.testing.assert_array_equal(space.vec.limits, znp.array([[[-1, -3]], [[1, 5]]]))
+    diffs = space.vec.upper - space.vec.lower
+    diffs_lim = space.vec.limits[1] - space.vec.limits[0]
+    np.testing.assert_array_equal(diffs, diffs_lim)
+
+    assert space.v1.volume == 16
+    assert space.vec.volume == znp.array([[16.]])
+
+
+def test_labels_space():
+
+    obs1 = zfit.Space("x", -1, 1)
+    ylabel = "y_label"
+    obs2 = zfit.Space("y", -3, 5, label=ylabel)
+    zlabel = "z_label"
+    obs3 = zfit.Space("z", -3, 5, label=zlabel)
+    obs4 = zfit.Space("w", -3, 5)
+
+    assert obs1.labels == ("x",)
+    assert obs1._labels == {"x": "x"}
+    assert obs2.labels == (ylabel,)
+    assert obs2._labels == {"y": ylabel}
+    assert obs3.labels == (zlabel,)
+    assert obs3._labels == {"z": zlabel}
+    assert obs4.labels == ("w",)
+    assert obs4._labels == {"w": "w"}
+
+
+    space = obs1 * obs2 * obs3 * obs4
+    assert space.labels == ("x", ylabel, zlabel, "w")
+    assert space._labels == {"x": "x", "y": ylabel, "z": zlabel, "w": "w"}
+
+    space243 = space.with_obs(["y", "w", "z"])
+    assert space243.labels == (ylabel, "w", zlabel)
